﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

using stdole;

using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.ArcMapUI;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Display;
using ESRI.ArcGIS.Geodatabase;
using ESRI.ArcGIS.Geometry;


namespace ValenceRenderer_CSharp
{
   
    class DrawNodeValenceSymbols
    {
        public IApplication m_pApp;
        private IMxDocument m_pMxDoc;
        private IMap m_pMap;

        private IFormattedTextSymbol[] m_pTextSymbol = new IFormattedTextSymbol[6];
        private short m_lOldActiveCache;

        public void DrawSymbols()
        {
            try
            {
                // Set up symbols
                SetupSymbols();

                // Get references to map document objects
                m_pMxDoc = (IMxDocument) m_pApp.Document;
                m_pMap = m_pMxDoc.FocusMap;

                // Get display properties
                IActiveView av = (IActiveView)m_pMap;
                IScreenDisplay sd = av.ScreenDisplay;
                IEnvelope env = av.Extent;
                ISpatialFilter sf = new SpatialFilter();
                sf.Geometry = env;
                sf.SearchOrder = esriSearchOrder.esriSearchOrderSpatial;
                sf.SpatialRel = esriSpatialRelEnum.esriSpatialRelEnvelopeIntersects;

                // Iterated through all map layers and render valence
                // for all visible simple junction classes
                
                IEnumLayer layers = m_pMap.get_Layers(null, true);
                ILayer layer = layers.Next();
                while (layer != null)
                {
                    if (layer is IGeoFeatureLayer && layer.Visible == true)
                    {
                        IGeoFeatureLayer gfLayer = (IGeoFeatureLayer)layer;
                        if (gfLayer.FeatureClass != null)
                        {
                            IFeatureClass fc = gfLayer.FeatureClass;
                            if (fc.FeatureType == esriFeatureType.esriFTSimpleJunction)
                            {
                                if (DrawValence.m_showGenericJunctions == true | IsGenericJunctionClass(fc) == false)
                                {
                                    sf.GeometryField = fc.ShapeFieldName;
                                    DrawSimpleJunctionFeatures(gfLayer, sf, sd);
                                }
                            }

                        }
                    }
                    layer = layers.Next();
                }

            }
            catch (Exception ex)
            {
            }
        }

        private bool IsGenericJunctionClass(IFeatureClass fc)
        {
            bool result = false;

            try
            {
                IDataset ds = (IDataset)fc;
                if (ds.Name.Contains("Net_Junctions"))
                    result = true;
            }
            catch
            { }

            return result;
        }

        private void SetupSymbols()
        {
            try
            {
                m_pTextSymbol[0] = FormattedTextSymbol(255, 0, 0, "0");
                m_pTextSymbol[1] = FormattedTextSymbol(0, 255, 0, "1");
                m_pTextSymbol[2] = FormattedTextSymbol(0, 0, 255, "2");
                m_pTextSymbol[3] = FormattedTextSymbol(255, 255, 0, "3");
                m_pTextSymbol[4] = FormattedTextSymbol(255, 0, 255, "4");
                m_pTextSymbol[5] = FormattedTextSymbol(255, 255, 255, "+");
            }
            catch (Exception ex)
            {
            }
        }

        private IFormattedTextSymbol FormattedTextSymbol(int r, int g, int b, string text)
        {
            IFormattedTextSymbol result = new TextSymbol();
            
            try
            {
                char letter = (char) text[0];
                int charIdx = (int)letter;

                double markerSize = DrawValence.m_markerSymbolSize;

                ICharacterMarkerSymbol cms = CreateCharacterMarkerSymbol(markerSize, 0.0, 0.0, 0.0, "ESRI Default Marker", 33, r, g, b);
                IMarkerTextBackground markerTextBackground = new MarkerTextBackground();
                IMultiLayerMarkerSymbol multiLayerMarkerSymbol = new MultiLayerMarkerSymbol();
                ILayerColorLock layerColorLock;

                multiLayerMarkerSymbol.AddLayer(cms);
                layerColorLock = (ILayerColorLock) multiLayerMarkerSymbol;
                layerColorLock.LayerColorLock[0] = true;

                markerTextBackground.Symbol = multiLayerMarkerSymbol;

                result.Color.RGB = 0;
                double textSize = DrawValence.m_markerTextSize;
                IFontDisp textFont = CreateFont("Arial", false, false, textSize);
                result.Font = textFont;
                result.HorizontalAlignment = esriTextHorizontalAlignment.esriTHALeft;
                result.VerticalAlignment = esriTextVerticalAlignment.esriTVATop;
                result.Background = markerTextBackground;

            }
            catch (Exception ex)
            {
            }

            return result;
        }

        private ICharacterMarkerSymbol CreateCharacterMarkerSymbol(double size, double angle, double xoffset, 
            double yoffset, string fontName, int charIdx, int r, int g, int b)
        {
            ICharacterMarkerSymbol result = new CharacterMarkerSymbol();

            try
            {
                result.Font = CreateFont(fontName, false, false, 2.0);
                result.Size = size;
                result.Angle = angle;
                result.Color = CreateColor(r, g, b, false);
                result.XOffset = xoffset;
                result.YOffset = yoffset;
                result.CharacterIndex = charIdx;
            }
            catch
            {
            }

            return result;
        }

        private IFontDisp CreateFont(string fontName, bool fontBold, bool fontItalic, double fontSize)
        {
            IFontDisp result = (IFontDisp) new StdFont();

            try
            {
                result.Name = fontName;
                result.Size = (decimal)fontSize;

                if (fontBold)
                    result.Bold = true;

                if (fontItalic)
                    result.Italic = true;
            }
            catch
            {
            }

            return result;
        }

        private IRgbColor CreateColor(int r, int g, int b, bool isNull)
        {
            IRgbColor result = new RgbColor();

            try
            {
                result.Red = r;
                result.Green = g;
                result.Blue = b;
                result.NullColor = isNull;
            }
            catch
            { }

            return result;
        }

        private void DrawSimpleJunctionFeatures(IGeoFeatureLayer gfl, ISpatialFilter sf, IScreenDisplay sd)
        {
            IFeatureCursor cur = null;
            try
            {
                // Loop through visble features
                m_lOldActiveCache = sd.ActiveCache;

                sd.ActiveCache = (short) esriScreenCache.esriNoScreenCache;
                sd.StartDrawing(0, (short) esriScreenCache.esriNoScreenCache);

                cur = gfl.SearchDisplayFeatures(sf, true);
                IFeature fea = cur.NextFeature();
                while (fea != null)
                {
                    DrawSimpleJunctionFeature(fea, sd);
                    fea = cur.NextFeature();
                }

                sd.FinishDrawing();
                sd.ActiveCache = m_lOldActiveCache;

            }
            catch (Exception ex)
            {
            }
            finally
            {
                ReleaseCOMObject(cur);
            }
        }

        private void DrawSimpleJunctionFeature(IFeature fea, IScreenDisplay sd)
        {
            try
            {
                if (fea is ISimpleJunctionFeature)
                {
                    ISimpleJunctionFeature sjf = (ISimpleJunctionFeature)fea;
                    int numEdges = sjf.EdgeFeatureCount;

                    sd.ScaleContents = true;

                    if (numEdges <= 4)
                        sd.SetSymbol((ISymbol)m_pTextSymbol[numEdges]);
                    else
                        sd.SetSymbol((ISymbol)m_pTextSymbol[5]);

                    string edges = Convert.ToString(numEdges);
                    sd.DrawText(fea.Shape, edges);


                }
            }
            catch (Exception ex)
            {
            }
        }

        private void ReleaseCOMObject(object obj)
        {
            try
            {
                Marshal.ReleaseComObject(obj);
            }
            catch { }
        }
    }
}
