VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ValencePropPage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True

' Copyright 2008 ESRI
' 
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
' 
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
' 
' See use restrictions at <your ArcGIS install location>/developerkit/userestrictions.txt.
' 




' class definition for ValencePropPage
' last update: (12/06/01)
'   1. added support for IComEmbeddedPropertyPage and
'      removed support for IPropertyPageContext.
'   2. also, no need to manually handling cloning of renderer, change
'      made to IComPropertyPage_SetObjects and IComEmbeddedPropertyPage_QueryObject
'      noted there by comments

Option Explicit

Private m_hDC As OLE_HANDLE
Private m_Page As frmValenceRenderer
Private m_pObjectRend As ValenceRend
Private m_Priority As Long

Implements IComPropertyPage
Implements IComEmbeddedPropertyPage
Implements IRendererPropertyPage

Private Sub Class_Initialize()
On Error GoTo ErrHand:
 
    Set m_Page = New frmValenceRenderer
    m_Priority = 610

    Exit Sub
ErrHand:
    MsgBox "ValencePropPage Initialize - " & Err.Description
    Exit Sub
End Sub

Private Sub Class_Terminate()
On Error GoTo ErrHand

    Set m_Page = Nothing
    
    Exit Sub
ErrHand:
    MsgBox "Class_Terminate - " & Err.Description
    Exit Sub
End Sub

Private Function IComPropertyPage_Activate() As esriSystem.OLE_HANDLE
' load form and return handle of Picture box
    
    Load m_Page
    IComPropertyPage_Activate = m_Page.picRenderer.hWnd
End Function

Private Function IComPropertyPage_Applies(ByVal Objects As esriSystem.ISet) As Boolean
' make sure we have an editable renderer object
On Error GoTo ErrHand
        
    Dim pObj As Variant
    
    If Objects.Count <= 0 Then
        IComPropertyPage_Applies = False
        Exit Function
    End If
    
    Objects.Reset
    Set pObj = Objects.Next
    Do While Not TypeOf pObj Is ValenceRend
        Set pObj = Objects.Next
        If pObj Is Nothing Then
            IComPropertyPage_Applies = False
            Exit Function
        End If
    Loop
    
    IComPropertyPage_Applies = True
        
    Exit Function
ErrHand:
    MsgBox "Applies - " & Err.Description
    Exit Function
End Function

Private Sub IComPropertyPage_Apply()
' When OK or Apply are pressed, call QueryObject and pass the page's renderer
On Error GoTo ErrHand:
        
    IComEmbeddedPropertyPage_QueryObject m_Page.GetRenderer
     
    Exit Sub
ErrHand:
    MsgBox "Apply - " & Err.Description
    Exit Sub
End Sub

Private Sub IComPropertyPage_Cancel()
' triggered when Cancel button is pressed
' doing nothing here discards any edits made to the page's renderer

End Sub

Private Sub IComPropertyPage_Deactivate()
' triggered when page loses focus.  unload page here

    Unload m_Page

End Sub

Private Property Get IComPropertyPage_Height() As Long
    IComPropertyPage_Height = m_Page.Height

End Property

Private Property Get IComPropertyPage_HelpContextID(ByVal controlID As Long) As Long
    '*' NOTIMPL
End Property

Private Property Get IComPropertyPage_HelpFile() As String
    '*' NOTIMPL
End Property

Private Sub IComPropertyPage_Hide()
    m_Page.picRenderer.Visible = False

End Sub

Private Property Get IComPropertyPage_IsPageDirty() As Boolean
' the framework checks this to see if page needs to be redrawn
    IComPropertyPage_IsPageDirty = m_Page.IsDirty

End Property

Private Property Set IComPropertyPage_PageSite(ByVal RHS As esriFramework.IComPropertyPageSite)
    m_Page.PageSite = RHS

End Property

Private Property Let IComPropertyPage_Priority(ByVal RHS As Long)

    If RHS > 0 Then
        m_Priority = RHS
    End If
End Property

Private Property Get IComPropertyPage_Priority() As Long
    
    IComPropertyPage_Priority = m_Priority
End Property

Private Sub IComPropertyPage_SetObjects(ByVal Objects As esriSystem.ISet)
' retrieve renderer from Objects list and pass a copy to the
'   page for editing
On Error GoTo ErrHand:

  Dim pObj As Variant
  Dim pRend As ValenceRend

  If Objects.Count <= 0 Then GoTo EndProc
  
  Objects.Reset
  Set pObj = Objects.Next
  
  'make sure we have the renderer object
  Do While (Not TypeOf pObj Is ValenceRend) And (Not pObj Is Nothing)
      Set pObj = Objects.Next
  Loop
  
  If TypeOf pObj Is ValenceRend Then
    
' cloning is handled by framework, so we don't have to do this
'    Set m_pObjectRend = pObj  ' hold on to the renderer
'
'    ' make a copy
'    Set pRend = New ValenceRend
'    pRend.Color0 = m_pObjectRend.Color0
'    pRend.Color1 = m_pObjectRend.Color1
'    pRend.Color2 = m_pObjectRend.Color2
'    pRend.Color3 = m_pObjectRend.Color3
'
'    m_Page.SetRenderer pRend ' pass the copy to the property page for editing
    
    m_Page.SetRenderer pObj
    
    m_Page.UpdateControls
    m_Page.ScaleMode = vbPixels
  End If

    GoTo EndProc
ErrHand:
    MsgBox "SetObjects - " & Err.Description
EndProc:
    Set pObj = Nothing
End Sub

Private Sub IComPropertyPage_Show()
    m_Page.picRenderer.Visible = True

End Sub

Private Property Get IComPropertyPage_Title() As String
    IComPropertyPage_Title = m_Page.Caption

End Property

Private Property Let IComPropertyPage_Title(ByVal strTitle As String)
    m_Page.Caption = strTitle

End Property

Private Property Get IComPropertyPage_Width() As Long
    IComPropertyPage_Width = m_Page.Width

End Property

Private Function IComEmbeddedPropertyPage_CreateCompatibleObject(ByVal kind As Variant) As Variant
' check to see if the renderer is compatible with the property page...
'   ... if so, return the renderer.  it not, create a new one.

    Dim pRend As ValenceRend
      
    If (TypeOf kind Is ValenceRend) And (Not kind Is Nothing) Then
        Set pRend = kind
    Else
        ' create a new MZ renderer
        Set pRend = New ValenceRend
    End If
    
    Set IComEmbeddedPropertyPage_CreateCompatibleObject = pRend
End Function

Private Sub IComEmbeddedPropertyPage_QueryObject(ByVal theObject As Variant)
' assign properties from the page's renderer to the permanent renderer
On Error GoTo ErrHand
    
    Dim pRend As ValenceRend
    
    If (TypeOf theObject Is ValenceRend) And (Not theObject Is Nothing) Then
' because cloning is handled by framework we don't have to do this either
'        Set pRend = theObject
'        m_pObjectRend.Color0 = pRend.Color0
'        m_pObjectRend.Color1 = pRend.Color1
'        m_pObjectRend.Color2 = pRend.Color2
'        m_pObjectRend.Color3 = pRend.Color3
    End If
    
    Exit Sub
ErrHand:
    MsgBox "QueryObject - " & Err.Description
    Exit Sub
End Sub

Private Function IRendererPropertyPage_CanEdit(ByVal obj As esriCarto.IFeatureRenderer) As Boolean
On Error GoTo ErrHand
    If TypeOf obj Is ValenceRend Then
        IRendererPropertyPage_CanEdit = True
    Else
        IRendererPropertyPage_CanEdit = False
    End If
    
    Exit Function

ErrHand:
    MsgBox "CanEdit - " & Err.Description
    Exit Function
End Function

Private Property Get IRendererPropertyPage_ClassID() As esriSystem.IUID
'GUID of the property page class
  
  Dim pUID As New UID
  
  pUID.Value = "ValenceRenderer.ValencePropPage"
  Set IRendererPropertyPage_ClassID = pUID
End Property

Private Property Get IRendererPropertyPage_Description() As String
    IRendererPropertyPage_Description = "Display number of edges connected to a junction."

End Property

Private Property Get IRendererPropertyPage_Name() As String
    IRendererPropertyPage_Name = "Valence Renderer"

End Property

Private Property Get IRendererPropertyPage_PreviewImage() As esriSystem.OLE_HANDLE
    IRendererPropertyPage_PreviewImage = m_Page.picPreview.Picture

End Property

Private Property Get IRendererPropertyPage_RendererClassID() As esriSystem.IUID
'GUID of the Renderer class
On Error GoTo ErrHand:
    Dim pUID As New UID
    pUID = "ValenceRenderer.ValenceRend"
    Set IRendererPropertyPage_RendererClassID = pUID
  
    GoTo EndProc
  
ErrHand:
    MsgBox "RendererClassID - " & Err.Description
EndProc:
    Set pUID = Nothing
End Property

Private Property Get IRendererPropertyPage_Type() As String
    IRendererPropertyPage_Type = "Network Renderers"

End Property
