VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ValenceRend"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True

' Copyright 2008 ESRI
' 
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
' 
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
' 
' See use restrictions at <your ArcGIS install location>/developerkit/userestrictions.txt.
' 




' class definition for Valence Renderer

' the ValenceRenderer is a custom feature renderer symbolizes simple
'   junction features with colored markers and positions labels according
'   to the number of connections
' the renderer implements ILegendInfo to support the TOC

' see ValencePropPage.cls for property page implementation

Option Explicit

Private m_LegendGroup As ILegendGroup
Const E_FAIL As Long = &H80004005  'Used to bail out of draw routine during drawPhases
                                   'besides geographic

Implements IFeatureRenderer
Implements IPersistVariant
Implements ILegendInfo

Private Sub Class_Initialize()
On Error GoTo ErrHand
    
    Set m_LegendGroup = New LegendGroup
    m_LegendGroup.Heading = "Number of Connections"
    
    Dim pLegendClass As ILegendClass
    Dim pMSym As IMarkerSymbol
    Dim pSMSym As ISimpleMarkerSymbol
    
    Dim pColors(4) As IRgbColor
    
    ' create and set default colors
    Set pColors(0) = New RgbColor
    pColors(0).RGB = 46465279   ' Pink
    Set pColors(1) = New RgbColor
    pColors(1).RGB = 255        ' Red
    Set pColors(2) = New RgbColor
    pColors(2).RGB = 16776960   ' Cyan
    Set pColors(3) = New RgbColor
    pColors(3).RGB = 32768      ' Dark Green
       
    ' create a legend class, create and assign symbol, then
    '   add to m_LegendGroup.  repeat.
    Dim i As Integer
    For i = 0 To 3
        Set pLegendClass = New LegendClass
        Set pSMSym = New SimpleMarkerSymbol
        ' initial symbol will be a simple marker symbol
        With pSMSym
            .Color = pColors(i)
            .Style = esriSMSSquare
            .Size = 6
        End With
        Set pMSym = pSMSym
        Set pLegendClass.Symbol = pMSym
        pLegendClass.Label = i
        m_LegendGroup.AddClass pLegendClass
    Next i
    
    ' special label for the last legend class
    m_LegendGroup.Class(3).Label = "3+"
    
    m_LegendGroup.Visible = True
    
    GoTo EndProc
ErrHand:
    MsgBox "ValenceRend Initialize - " & Err.Description
EndProc:
    Set pLegendClass = Nothing
    Set pMSym = Nothing
    Set pSMSym = Nothing
    Set pColors(0) = Nothing
    Set pColors(1) = Nothing
    Set pColors(2) = Nothing
    Set pColors(3) = Nothing
    Exit Sub
End Sub

Private Sub Class_Terminate()
    Set m_LegendGroup = Nothing
End Sub

Private Function IFeatureRenderer_CanRender(ByVal featClass As esriGeoDatabase.IFeatureClass, ByVal Display As esriDisplay.IDisplay) As Boolean
On Error GoTo ErrHand:
  
  'Only use this renderer if the feature class contains simple junctions
  If featClass.FeatureType = esriFTSimpleJunction Then
    IFeatureRenderer_CanRender = True
  Else
    IFeatureRenderer_CanRender = False
  End If
  
  Exit Function
  
ErrHand:
    MsgBox "IFeatureRenderer CanRender - " & Err.Description
    Exit Function
End Function

Private Sub IFeatureRenderer_Draw(ByVal Cursor As esriGeoDatabase.IFeatureCursor, ByVal drawPhase As esriSystem.esriDrawPhase, ByVal Display As esriDisplay.IDisplay, ByVal trackCancel As esriSystem.ITrackCancel)
' loop through the features and draw them
On Error GoTo ErrHand:
     
  Dim pFeat As IFeature
  Dim conCount As Long
  Dim pJunctionFeat As ISimpleJunctionFeature
  Dim pMSym As IMarkerSymbol
  Dim pFeatDraw As IFeatureDraw
  Dim bContinue As Boolean
    
  If Display Is Nothing Then
    GoTo EndProc
  End If
    
  If drawPhase <> esriDPGeography Then
    'If we are not drawing geography, then bail out of the draw routine.  This insures
    'that feature using this renderer will draw with the default symbology when selected
    On Error GoTo 0
    Err.Raise E_FAIL
  
  Else
      Set pFeat = Cursor.NextFeature
      bContinue = True
      
      ' while there are still more features and drawing has not been canceled
      Do While (Not pFeat Is Nothing) And (bContinue = True)
        Set pJunctionFeat = pFeat
        conCount = pJunctionFeat.EdgeFeatureCount
        ' set symbols based on number of connections
        Select Case conCount
        Case 0
          Set pMSym = m_LegendGroup.Class(0).Symbol
        Case 1
          Set pMSym = m_LegendGroup.Class(1).Symbol
        Case 2
          Set pMSym = m_LegendGroup.Class(2).Symbol
        Case Else
          Set pMSym = m_LegendGroup.Class(3).Symbol
        End Select
    
        Set pFeatDraw = pFeat
        Display.SetSymbol pMSym
        pFeatDraw.Draw drawPhase, Display, pMSym, True, Nothing, esriDSNormal
        DrawMyText pFeat, Display, conCount, pMSym.Color
        
        Set pFeat = Cursor.NextFeature
        If Not trackCancel Is Nothing Then bContinue = trackCancel.Continue
      Loop
      
  End If
      
  GoTo EndProc
  
ErrHand:
    MsgBox "Draw - " & Err.Description
EndProc:
    Set pFeat = Nothing
    Set pJunctionFeat = Nothing
    Set pMSym = Nothing
    Set pFeatDraw = Nothing
    Exit Sub
End Sub

Private Sub DrawMyText(inFeature As IFeature, Display As IDisplay, conCount As Long, txtColor As IColor)
' draws label on map as colored text
' position and color depend on the number of connections at the feature
On Error GoTo ErrHand:
  
    Dim pTSym As ISimpleTextSymbol
    Set pTSym = New TextSymbol
    Dim fnt As New StdFont
    fnt.Name = "Arial"
    fnt.Size = 12
      
    Select Case conCount
    Case 0
        pTSym.HorizontalAlignment = esriTHARight
        pTSym.VerticalAlignment = esriTVABottom
    Case 1
        pTSym.HorizontalAlignment = esriTHALeft
        pTSym.VerticalAlignment = esriTVABottom
    Case 2
        pTSym.HorizontalAlignment = esriTHARight
        pTSym.VerticalAlignment = esriTVATop
    Case Else
        pTSym.HorizontalAlignment = esriTHALeft
        pTSym.VerticalAlignment = esriTVATop
    End Select
  
    pTSym.Font = fnt
    pTSym.Color = txtColor
    With Display
        .SetSymbol pTSym
        .DrawText inFeature.Shape, CStr(conCount)
    End With

    GoTo EndProc
  
ErrHand:
    MsgBox "DrawText - " & Err.Description
EndProc:
    Set pTSym = Nothing
    Set fnt = Nothing
    Exit Sub
End Sub

Private Property Set IFeatureRenderer_ExclusionSet(ByVal RHS As esriCarto.IFeatureIDSet)

End Property

Private Sub IFeatureRenderer_PrepareFilter(ByVal fc As esriGeoDatabase.IFeatureClass, ByVal QueryFilter As esriGeoDatabase.IQueryFilter)
On Error GoTo ErrHand:
  
    QueryFilter.AddField fc.OIDFieldName
    ' no other fields added because symbology is not dependent on field values
  
    Exit Sub

ErrHand:
    MsgBox "PrepareFilter - " & Err.Description
    Exit Sub
End Sub

Private Property Get IFeatureRenderer_RenderPhase(ByVal drawPhase As esriSystem.esriDrawPhase) As Boolean
    If drawPhase = esriDPGeography Then
        IFeatureRenderer_RenderPhase = True
    Else
        IFeatureRenderer_RenderPhase = False
    End If
End Property

Private Property Get IFeatureRenderer_SymbolByFeature(ByVal Feature As esriGeoDatabase.IFeature) As esriDisplay.ISymbol

End Property

Private Property Get ILegendInfo_LegendGroup(ByVal Index As Long) As esriCarto.ILegendGroup
    Set ILegendInfo_LegendGroup = m_LegendGroup
End Property

Private Property Get ILegendInfo_LegendGroupCount() As Long
    If m_LegendGroup Is Nothing Then
        ILegendInfo_LegendGroupCount = 0
    Else
        ILegendInfo_LegendGroupCount = 1
    End If
End Property

Private Property Get ILegendInfo_LegendItem() As esriCarto.ILegendItem
    ' no legend item
    Set ILegendInfo_LegendItem = Nothing
End Property

Private Property Let ILegendInfo_SymbolsAreGraduated(ByVal RHS As Boolean)
    
End Property

Private Property Get ILegendInfo_SymbolsAreGraduated() As Boolean
    ILegendInfo_SymbolsAreGraduated = False
End Property

Private Property Get IPersistVariant_ID() As esriSystem.IUID
On Error GoTo ErrHand:
  
    Dim pUID As New UID
    pUID = "ValenceRenderer.ValenceRend"

    Set IPersistVariant_ID = pUID

    GoTo EndProc

ErrHand:
    MsgBox "ID - " & Err.Description
    Exit Property
EndProc:
    Set pUID = Nothing
End Property

Private Sub IPersistVariant_Load(ByVal Stream As esriSystem.IVariantStream)
'Load the persisted parameters of the renderer
    
    Set m_LegendGroup = Stream.Read
End Sub

Private Sub IPersistVariant_Save(ByVal Stream As esriSystem.IVariantStream)
'Persist the settings for the renderer
    
    Stream.Write m_LegendGroup
End Sub

Public Property Get Color0() As IRgbColor
On Error GoTo ErrHand
   
    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(0).Symbol
    Set Color0 = pIMSym.Color
    
    GoTo EndProc
  
ErrHand:
    MsgBox "Get Color0 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Let Color0(ByVal inColor As IRgbColor)
On Error GoTo ErrHand:
   
    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(0).Symbol
    pIMSym.Color = inColor
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Let Color0 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Get Color1() As IRgbColor
On Error GoTo ErrHand:
    
    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(1).Symbol
    Set Color1 = pIMSym.Color
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Get Color1 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Let Color1(ByVal inColor As IRgbColor)
On Error GoTo ErrHand:
    
    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(1).Symbol
    pIMSym.Color = inColor
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Let Color1 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Get Color2() As IRgbColor
On Error GoTo ErrHand:

    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(2).Symbol
    Set Color2 = pIMSym.Color
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Get Color2 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Let Color2(ByVal inColor As IRgbColor)
On Error GoTo ErrHand:
    
    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(2).Symbol
    pIMSym.Color = inColor
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Let Color2 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Get Color3() As IRgbColor
On Error GoTo ErrHand:
   
    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(3).Symbol
    Set Color3 = pIMSym.Color
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Get Color3 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property

Public Property Let Color3(ByVal inColor As IRgbColor)
On Error GoTo ErrHand:

    Dim pIMSym As IMarkerSymbol
    Set pIMSym = m_LegendGroup.Class(3).Symbol
    pIMSym.Color = inColor
  
    GoTo EndProc
  
ErrHand:
    MsgBox "Let Color3 - " & Err.Description
EndProc:
    Set pIMSym = Nothing
    Exit Property
End Property


